from ._build import build_python_language, py_language, py_parser
from ._versions import __grammar_version__, __version__

__all__ = [
    "__version__",
    "__grammar_version__",
    "py_language",
    "py_parser",
    "build_python_language",
]



from pathlib import Path
from threading import Semaphore
from typing import Optional, Union

from tree_sitter import Language, Parser

_PACKAGE = Path(__file__).parent

_BUILD_INPUT = _PACKAGE / "src"
_BUILD_TARGET = "python-language.so"

_language_cache: Optional[Language] = None
_parser_cache: Optional[Parser] = None


def py_parser(rebuild: bool = False) -> Parser:
    global _parser_cache
    if _parser_cache is None:
        _parser_cache = Parser()
        _parser_cache.set_language(py_language(rebuild=rebuild))
    assert isinstance(_parser_cache, Parser), "parser must not be None by now"
    return _parser_cache


def py_language(rebuild: bool = False) -> Language:
    global _language_cache
    if rebuild or not _language_cache:
        _language_cache = Language(
            build_python_language(rebuild=rebuild, build_lib=_PACKAGE),
            "python",
        )
    assert isinstance(_language_cache, Language), "language must not be None by now"
    return _language_cache


def build_python_language(
    rebuild: bool = False,
    build_lib: Union[str, Path] = _PACKAGE,
    _lock: Semaphore = Semaphore(),
) -> Path:
    """compile the python language into a useable .so file"""
    assert _BUILD_INPUT.exists(), "the language files must be downloaded"

    target = Path(build_lib) / _BUILD_TARGET

    if rebuild and target.exists():
        target.unlink()

    with _lock:
        if not target.exists():
            build_successful = Language.build_library(str(target), [str(_PACKAGE)])
            assert build_successful, "python tree-parser language failed to build"

    return target



import argparse
import sys
from pathlib import Path
from time import time

sys.path.append(str(Path(__file__).parent))


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--version",
        help="print version of this package",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--language-version",
        help="prints version of the tree-sitter-python-language that is being compiled",
        action="store_true",
        default=False,
    )

    return parser.parse_args()


def main():
    args = parse_args()
    if args.version:
        from _versions import __version__

        print(__version__)

    elif args.language_version:
        from _versions import __grammar_version__

        print(__grammar_version__)

    else:
        start = time()
        from _build import py_language

        py_language(rebuild=True)
        print(f"Recompiled language in {time() - start:0.4} seconds")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import sys
from cmath import log
from distutils import log as logger
from distutils.command.build_ext import build_ext
from distutils.core import Extension, setup
from glob import glob
from pathlib import Path
from pprint import pformat
from typing import Any, Dict

package_name = "tree-sitter-python-language"
package_under = package_name.replace("-", "_")
package_library = "python-language.so"

# add tree_sitter_python_language to prevent __init__ from running
sys.path.append(str(Path(__file__).parent / package_under))

try:
    from _versions import __grammar_version__, __version__
except ModuleNotFoundError as e:
    raise ModuleNotFoundError("Run 'make build' before using setup.py") from e

packages = [package_under]

package_data = {
    "": ["*"],
    package_under: [package_library, "src/*", "src/tree_sitter/*"],
}

requirements = ["tree_sitter>=0.20.0,<0.21.0"]


tree_sitter_extension = Extension(
    "tree_sitter_builder",
    # sources=[str(_) for _ in (Path(package_under) / "src").glob("**")],
    sources=[],
    libraries=[package_library],
)


class tree_sitter_build_ext(build_ext, object):
    """
    Specialized builder for testlib library

    """

    def build_extension(self, ext: Extension):
        if ext is tree_sitter_extension:
            self.build_tree_sitter_extension(ext)
        else:
            super().build_extension(ext)

    def build_tree_sitter_extension(self, ext: Extension):
        logger.info("Building python-language.so")
        from tree_sitter_python_language import build_python_language

        build_python_language(
            rebuild=True, build_lib=Path(self.build_lib) / package_under
        )


setup_kwargs: Dict[str, Any] = {
    "name": package_name,
    "version": __version__,
    "description": f"A pip-installable version of {package_name}-v{__grammar_version__}",
    "long_description": None,
    "author": "adam",
    "author_email": None,
    "maintainer": None,
    "maintainer_email": None,
    "url": "https://github.com/NorthIsUp/tree-sitter-python-language",
    "packages": packages,
    "package_data": package_data,
    "setup_requires": requirements,
    "install_requires": requirements,
    "python_requires": ">=3.7,<4.0",
    "ext_modules": [tree_sitter_extension],
    "cmdclass": {"build_ext": tree_sitter_build_ext},
}

setup(**setup_kwargs)



# This file is autogenerated by make
# values can be changed in the Makefile

# Version of this python package, used in setup.py
__version__ = "0.14.1"

# Version of the grammar pulled down during packaging
#   https://github.com/tree-sitter/tree-sitter-python/archive/refs/tags/v0.19.0.tar.gz
__grammar_version__ = "0.19.0"
